@@ -12,6 +12,11 @@ using namespace glmmr;
 template<typename modeltype>
 class rtsModelOptim : public ModelOptim<modeltype> {  public:
     
+    // this is stupid - it has to be re-added here because of the way CRAN forces an update
+    // will be removed once both packages are updated
+    std::pair<double,double>          rts_current_ll_var = {0.0,0.0};
+    std::pair<double,double>          rts_previous_ll_var = {0.0,0.0};
+    
     rtsModelOptim(modeltype& model_,  glmmr::ModelMatrix<modeltype>& matrix_, glmmr::RandomEffects<modeltype>& re_) : ModelOptim<modeltype>(model_,matrix_,re_) {};    
     rtsModelOptim(const rts::rtsModelOptim<modeltype>& optim) : ModelOptim<modeltype>(optim.model,optim.matrix,optim.re) {};    
     
@@ -24,6 +29,8 @@ class rtsModelOptim : public ModelOptim<modeltype> {  public:
     void            ml_theta();
     template<class algo, typename = std::enable_if_t<std::is_base_of<optim_algo, algo>::value> >
     void            ml_rho();
+    // remove this function when CRAN is updated as base class has it defined...
+    double          ll_diff_variance(bool beta = true, bool theta = true);
     double          log_likelihood_rho(const dblvec &rho);
     double          log_likelihood_rho_with_gradient(const VectorXd &rho, VectorXd& g);
     double          log_likelihood_beta(const dblvec &beta);
@@ -42,9 +49,7 @@ inline void rts::rtsModelOptim<modeltype>::ml_beta()
   dblvec start = this->get_start_values(true,false,false);
   // store previous log likelihood values for convergence calculations
   this->previous_ll_values.first = this->current_ll_values.first;
-  if(this->ll_previous.rows() != this->ll_current.rows()) this->ll_previous.resize(this->ll_current.rows(),NoChange);
-  double old_ll = this->log_likelihood_beta(start);
-  this->ll_previous.col(0) = this->ll_current.col(0);
+  rts_previous_ll_var.first = rts_current_ll_var.first;
 
   if constexpr (std::is_same_v<algo,LBFGS>){
     VectorXd start_vec = Map<VectorXd>(start.data(),start.size());
@@ -82,6 +87,8 @@ inline void rts::rtsModelOptim<modeltype>::ml_beta()
 
   int eval_size = this->control.saem ? this->re.mcmc_block_size : this->ll_current.rows();
   this->current_ll_values.first = this->ll_current.col(0).tail(eval_size).mean();
+  rts_current_ll_var.first = (this->ll_current.col(0).tail(eval_size) - this->ll_current.col(0).tail(eval_size).mean()).square().sum() / (eval_size - 1);
+  
 }
 
 template<typename modeltype>
@@ -89,7 +96,7 @@ inline double rts::rtsModelOptim<modeltype>::log_likelihood_beta(const dblvec& b
 {
   this->model.linear_predictor.update_parameters(beta);
   double ll = this->log_likelihood();
-  this->fn_counter.first += this->model.n() * this->re.scaled_u_.cols();
+  this->fn_counter.first += this->re.scaled_u_.cols();
   if(this->control.saem)
   {
     int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
@@ -161,13 +168,9 @@ inline void rts::rtsModelOptim<modeltype>::ml_theta()
   if(this->re.scaled_u_.cols() != this->re.u_.cols())this->re.scaled_u_.resize(NoChange,this->re.u_.cols());
   this->re.scaled_u_ = this->model.covariance.Lu(this->re.u_);  
 
-  if(this->ll_previous.rows() != this->ll_current.rows()) this->ll_previous.resize(this->ll_current.rows(),NoChange);
   this->previous_ll_values.second = this->current_ll_values.second;
-  double old_ll = this->log_likelihood_theta(start);
-  this->ll_previous.col(1) = this->ll_current.col(1);
+  rts_previous_ll_var.second = rts_current_ll_var.second;
 
-  if(this->re.scaled_u_.cols() != this->re.u_.cols())this->re.scaled_u_.conservativeResize(NoChange,this->re.u_.cols());
-  this->re.scaled_u_ = this->model.covariance.Lu(this->re.u_);  
   if constexpr (std::is_same_v<algo,LBFGS>){
     VectorXd start_vec = Map<VectorXd>(start.data(),start.size());
     optim<double(const VectorXd&, VectorXd&),algo> op(start_vec); 
@@ -212,6 +215,8 @@ inline void rts::rtsModelOptim<modeltype>::ml_theta()
   {
     int eval_size = this->control.saem ? this->re.mcmc_block_size : this->ll_current.rows();
     this->current_ll_values.second = this->ll_current.col(1).tail(eval_size).mean();
+    rts_current_ll_var.second = (this->ll_current.col(1).tail(eval_size) - this->ll_current.col(1).tail(eval_size).mean()).square().sum() / (eval_size - 1);
+    
   }  
 }
 
@@ -265,6 +270,7 @@ inline void rts::rtsModelOptim<modeltype>::ml_rho()
 
   int eval_size = this->control.saem ? this->re.mcmc_block_size : this->ll_current.rows();
   this->current_ll_values.second = this->ll_current.col(1).tail(eval_size).mean();
+  rts_current_ll_var.second = (this->ll_current.col(1).tail(eval_size) - this->ll_current.col(1).tail(eval_size).mean()).square().sum() / (eval_size - 1);
 }
 
 template<typename modeltype>
@@ -278,7 +284,7 @@ template<typename modeltype>
 inline double rts::rtsModelOptim<modeltype>::log_likelihood_theta(const dblvec& theta)
 {
   this->model.covariance.update_parameters(theta);
-  this->fn_counter.second += this->Q() * this->re.scaled_u_.cols();
+  this->fn_counter.second += this->re.scaled_u_.cols();
 #pragma omp parallel
   for(int i = 0; i < this->re.scaled_u_.cols(); i++)
   {
@@ -323,7 +329,7 @@ inline double rts::rtsModelOptim<modeltype>::log_likelihood_theta(const dblvec&
 template<>
 inline double rts::rtsModelOptim<BitsHSGP>::log_likelihood_theta(const dblvec& theta){
   this->model.covariance.update_parameters(theta);
-  this->fn_counter.second += this->Q() * this->re.scaled_u_.cols();
+  this->fn_counter.second += this->re.scaled_u_.cols();
 #pragma omp parallel
   for(int i = 0; i < this->re.scaled_u_.cols(); i++)
   {
@@ -369,7 +375,7 @@ template<typename modeltype>
 inline double rts::rtsModelOptim<modeltype>::log_likelihood_rho(const dblvec& rho)
 {
   this->model.covariance.update_rho(rho[0]);
-  this->fn_counter.second += this->Q() * this->re.scaled_u_.cols();
+  this->fn_counter.second += this->re.scaled_u_.cols();
 #pragma omp parallel
   for(int i = 0; i < this->re.scaled_u_.cols(); i++)
   {
@@ -417,7 +423,7 @@ inline double rts::rtsModelOptim<BitsHSGP>::log_likelihood_rho(const dblvec& rho
   this->model.covariance.update_rho(rho[0]);
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
   double ll = this->log_likelihood(false);
-  this->fn_counter.first += this->model.n() * this->re.scaled_u_.cols();
+  this->fn_counter.first += this->re.scaled_u_.cols();
   if(this->control.saem)
   {
     int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
@@ -560,6 +566,15 @@ inline double rts::rtsModelOptim<BitsHSGP>::log_likelihood_rho_with_gradient(con
   }
 }
 
+template<typename modeltype>
+inline double rts::rtsModelOptim<modeltype>::ll_diff_variance(bool beta, bool theta)
+{
+  double var = 0;
+  if(beta) var += rts_current_ll_var.first + rts_previous_ll_var.first;
+  if(theta) var += rts_current_ll_var.second + rts_previous_ll_var.second;
+  return var ; 
+}
+
 template<typename modeltype>
 inline void rts::rtsModelOptim<modeltype>::update_u(const MatrixXd& u_, bool append)
 {
@@ -575,9 +590,9 @@ inline void rts::rtsModelOptim<modeltype>::update_u(const MatrixXd& u_, bool app
     if(u_.cols()!=this->re.u_.cols()){
       this->re.u_.resize(NoChange,newcolsize);
       this->re.zu_.resize(NoChange,newcolsize);
-      this->re.u_ = u_;
-      if(newcolsize != this->ll_current.rows()) this->ll_current.resize(newcolsize,NoChange);
     }
+    this->re.u_ = u_;
+    if(newcolsize != this->ll_current.rows()) this->ll_current.resize(newcolsize,NoChange);
   }
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
 }
